
/*

    display.c

    - reads messages from queue and displays

    TODO:

    - add raw dump support (suitable to be loaded into spreadsheet, gnuplot)

    Author: Jason Moore, 1999

*/

#include <config.h>

#include "display.h"

// i need this a stupid global (int)display_alive that is defined in main.. //
int display_alive;

void display_init(void)
{
    initscr();            // initial ncurses screen/window
    noecho();            // don't show keystrokes
    cbreak();            // makes getch() returns true if a key is pressed 

    /*
    printf("Thr  Cnt pri flag           time       min       max       avg  bytes\n");
    printf("---------------------------------------------------------------------\n");
    */

    mvprintw(0,0,"[%s v%s] - %s", APP_NAME, VERSION, APP_DESC);
    // mvprintw(2,1,"server: %s port: %s config:", "-", "-", "-");
    mvprintw(3,0,"thr cnt code         time       min       max       avg   bytes");
    mvprintw(4,0,"---------------------------------------------------------------");
    refresh();
}


void display_exit(void)
{
    endwin();            // end ncurses screen
}



void display_update(void)
/*
 *    Update the screen with a message from the message queue.
 *    This method is triggered by the thread condition "new_message"
 * 
 *    just writes whole time-status messages to the screen - but
 *    it may need to be extended to write different types of messages.
 *
 *    MSG_HTTP_STATUS
 *    MSG_ERROR
 *
 *
 */
{
    struct msg *m;

    char flag1[2];
    char flag2[2];
    
    while (display_alive == 1)       /* global flag */
    {

        while (msg_unshift(&m) == 0)
        {

            /* error flag */   

            if (m->info & MSG_USER_ERROR)
                strcpy(flag2, DISPLAY_USER_ERROR);
            else if (m->info & MSG_SERVER_ERROR)
                strcpy(flag2, DISPLAY_SERVER_ERROR);
            else
                strcpy(flag2, DISPLAY_BLANK);

            /* min-max flag */   

            if (m->info & MSG_MIN)
                strcpy(flag1, DISPLAY_MIN);
            else if (m->info & MSG_MAX)
                strcpy(flag1, DISPLAY_MAX);
            else
                strcpy(flag1, DISPLAY_BLANK);

            mvprintw((m->thread_id+5), 0, "%2d  %3d %-3s  %1s%1s  %8s  %8s  %8s  %8s  %6d", 
                    m->thread_id, m->counter, m->msg,
                    flag1, flag2,
                    m->t, m->t_min, m->t_max, m->t_avg, m->bytes );

            refresh();
        }

        #ifdef DEBUG
        fprintf(stderr, "About to do conditional wait for \"new_message\"\n");
        #endif

        pthread_mutex_lock(&display_cond_lock);            // locks are globally defined.. 

        pthread_cond_wait(&new_message, &display_cond_lock);
        
        pthread_mutex_unlock(&display_cond_lock);
        
    }    

}


void display_status(char *msg)
/*
 * writes a one line message to the screen 
 * keeps a few messages on the screen before scrolling off.
 *
 * writes message to bottom line of the screen
 *
 * TODO: keep status msg buffer [16 lines] (scroll?) (msg counter?)
 *
 */
{
    // screen size - stored in ints LINES, COLS

    // clear line first before writing

    mvprintw((LINES-1), 0, "                                                              ");
    mvprintw((LINES-1), 0, msg);
    refresh();
}


//
// prints message in header
//

void display_status_header (char *msg, int line)
{
    if (line < 0 || line > 4)
    {
        fprintf(stderr, "display_status_header: bad line number: %d\n", line);
        return;        
    }

    mvprintw(line,0, "%s", msg);

}



