
/* global.h */

#ifndef _GLOBAL_H
#define _GLOBAL_H


#if HAVE_SYS_TIME_H
#include <sys/time.h>
#endif

#include <pthread.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
//#include <string.h>        // bzero
#include <fcntl.h>
#include <netdb.h>

#include "url.h"

#define MAXLINE                     1024
#define MAX_THREADS                 64
#define DEFAULT_NUMBER_THREADS      4
#define DEFAULT_NUMBER_REQUESTS     5
#define DEFAULT_DELAY               5
#define DEFAULT_HTTP_PORT           80
#define HTTP_REQUEST_SIZE           3            // kb
#define HTTP_RESPONSE_SIZE          8            // kb

#define APP_URL                     "<http://www.sober.com/wload/>"
#define APP_NAME                    "wload"
#define APP_DESC                    "curses-based webserver load tester"
// VERSION is defined in configure.in

#define OPTION_RESTART              1
#define OPTION_EXIT                 2

struct http_request {
    int thread_id;                      // unique thread id
    int counter;                        // number of requests 
    struct timeval min;                 // minimum time of http request
    struct timeval max;                 // max
    struct timeval avg;                 // avg
    struct timeval start;               // start time of request
    struct timeval end;                 // end
    struct timeval dur;                 // duration
    //struct msg m;                     // msg to create to pass to display
    char status[4];                     // http status code (200, 500, etc.) 3 bytes + \0
    short flag;                         // flags: error, ...
    struct url u;                       // url of request (host, port, etc.)
    int trigger;                        // flag to trigger starting (used with cond variable) (unused?)
    int bytes;                          // bytes received
    char req[HTTP_REQUEST_SIZE*1024];   // actual data of http request
    FILE *fh;                           // filehandle for saving response

    // reponse is not stored, but it can be saved to a file as it is being read
    // char res[HTTP_RESPONSE_SIZE*1024];  // http response           
};


/* prototypes */

int build_request(char *url_string);
int http_request(struct http_request *);
int _load_http_request(char *file_name, char *g_buffer, struct url *url);
void worker_main(struct http_request *);
int option_end(void);
void show_usage(void);


/* global variables */

struct http_request requests[MAX_THREADS];             // data each thread needs 
pthread_t workers[MAX_THREADS];                        // threads that do the work
pthread_t display_thread;

// char g_request[HTTP_REQUEST_SIZE*1024];             // global request struct (used to seed requests[])

pthread_cond_t slave_driver;//    = PTHREAD_COND_INITIALIZER;   // crack the whip
pthread_cond_t new_message;//    = PTHREAD_COND_INITIALIZER;    // signal new message to the waiting message displayer 

/*
    need mutex for any of the following non-thread safe functions:
        - inet_aton     
        - gethost*
        - getnet*
        - getproto*
        - getserv*
        - inet_ntoa
        (source: stevens, unix network programming p. 611)

    Q: is "gettimeofday" thread-safe? 

*/

int thread_id;            // each thread uses this number to get its id. 
// struct url g_url;        // global URL struct which all threads can copy from 

extern int display_alive;

// mutex for locking msg display function conditional
pthread_mutex_t display_cond_lock;//    = PTHREAD_MUTEX_INITIALIZER;

#endif

