
/* msg.h */
 
//    
//  - data structure to hold messages, and queue
//    them for being painted on the screen.
//
//  - a message is used to pass data to a thread
//    that reads messages and displays them on the screen.
//


/*

Copyright (C) 1999 Jason Moore <jmoore@sober.com>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


#ifndef _MSG_H
#define _MSG_H

#define MAX_QUEUE_LENGTH 64

                					// size of array - if messages keep 
			                		// getting created, then they will 
	                				// overwrite old ones

/* info in http_request holds these possible flags */

#define MSG_MIN             1
#define MSG_MAX             2
#define MSG_USER_ERROR      4
#define MSG_SERVER_ERROR    8
#define MSG_IMPORTANT       16
#define SAVE_RESPONSE       32

/* message structure that is passed to the display thread */


struct msg {
	int	thread_id;	    	// unique thread id
	int	counter;	    	// counts number of hits
	short 	info;	    	// extra info on msg
	char	msg[9];			// 404/500/401/etc
	char	t[9];			// SSS.MMMM\0 [9]
	char	t_min[9];
	char	t_max[9];
	char	t_avg[9];
	int	bytes;
};

struct msg_queue {
	int reader;			// where displayer is reading from
	int writer;			// where messages are being written to
	struct msg messages[MAX_QUEUE_LENGTH];
};


int msg_init();                     // wants a pointer.. it gets memory

int msg_push(struct msg *);         // msgs added by anyone (worker threads) 

int msg_unshift(struct msg **);		// only called by displayer


#endif


