
/* time_it.c */

/*

    - handle time and time calculations.
    - (formerly time.c, but renamed to not conflict with stanard time.h)

*/

#include <config.h>

#include "time_it.h"

// min/max for all threads
// struct timeval g_min, g_max;


// calculates the difference between 2 timevals and puts value in the third

int time_diff(struct timeval *start, struct timeval *end, struct timeval *diff)
{
    // [seconds].[useconds*million]
    
    if (end->tv_usec < start->tv_usec)        // e.g. start: 2.4 end: 5.1 (carry the...)
    {
        diff->tv_sec  = (end->tv_sec - 1) - start->tv_sec;
        diff->tv_usec = (end->tv_usec + 1000000) - start->tv_usec; 
    }
    else
    {
        diff->tv_sec    = (end->tv_sec - start->tv_sec); 
        diff->tv_usec    = (end->tv_usec - start->tv_usec);
    }

    return 0;
}


//
int timestr(struct timeval *tv, char *msg_buff)
//
//   convert timeval structure tv to a string and copy into to msg_buff 
//
{
    // lower resolution of micro secs (10**6 -> 10**4), pad micros with zeros and prepend secs
    // e.g. 168.0068

    sprintf(msg_buff, "%d.%04d", (int)tv->tv_sec, (int)(tv->tv_usec/100));

    return 0;
}


int time_update_min(struct timeval *min, struct timeval *dur)
{
    /* check if the new duration is a new minimum.. */

    if (min->tv_sec == 0 && min->tv_usec == 0) {
        min->tv_sec  = dur->tv_sec;
        min->tv_usec = dur->tv_usec;
        return 0;        
    }    

    if (min->tv_sec < dur->tv_sec) {    // no change    
        return 0;        
    } 

    if (min->tv_sec > dur->tv_sec) {
        min->tv_sec  = dur->tv_sec;
        min->tv_usec = dur->tv_usec;
        return 0;
    }

    if (min->tv_usec < dur->tv_usec) {
        return 0;
    }

    #ifdef DEBUG
    fprintf(stderr, "min:%d dur:%d min gets updated\n", (int)min->tv_usec, (int)dur->tv_usec);
    #endif
    min->tv_sec  = dur->tv_sec;
    min->tv_usec = dur->tv_usec;

    return 0;
} 


int time_update_max(struct timeval *max, struct timeval *dur)
{
    /* check if the new duration is a new maximum.. */

    if (max->tv_sec == 0 && max->tv_usec == 0) {
        max->tv_sec  = dur->tv_sec;
        max->tv_usec = dur->tv_usec;
        return 0;        
    }    

    if (max->tv_sec > dur->tv_sec) {    // no change    
        return 0;        
    } 

    if (max->tv_sec < dur->tv_sec) {
        max->tv_sec  = dur->tv_sec;
        max->tv_usec = dur->tv_usec;
        return 0;
    }

    // they are equal.. check usecs now.

    if (max->tv_usec > dur->tv_usec) {
        return 0;
    }

    max->tv_sec  = dur->tv_sec;
    max->tv_usec = dur->tv_usec;

    return 0;
} 

int time_update_avg(struct timeval *avg, int count, struct timeval *dur)
{
    /* add the new duration to the existing average */

    // note: count has already been incremented to current count
    //       

    long t_usec = 0;                    // tmp

    // calculate sum 
    t_usec =  (avg->tv_usec * (count-1));
    t_usec += (avg->tv_sec  * (count-1))*1000000;

    t_usec += dur->tv_sec*1000000;      // add
    t_usec += dur->tv_usec;

    #ifdef DEBUG
    fprintf(stderr, "t_usec:%l  count:%d (avg totals)\n", t_usec, count);
    #endif

    t_usec /= count;                // update

    avg->tv_sec = t_usec/1000000;   // assign
    avg->tv_usec = t_usec%1000000;

    #ifdef DEBUG
    fprintf(stderr, "avg:%d dur:%d min gets updated\n", (int)min->tv_usec, (int)dur->tv_usec);
    #endif

    return 0;
} 

